@@ -25,7 +25,7 @@
 
 #include <fnmatch.h>
 #include <getopt.h>
-#include "busybox.h"
+#include "libbb.h"
 #include "unarchive.h"
 
 #if ENABLE_FEATURE_TAR_CREATE
@@ -134,7 +134,6 @@ static void freeHardLinkInfo(HardLinkInfo ** hlInfoHeadPtr)
 		}
 		*hlInfoHeadPtr = NULL;
 	}
-	return;
 }
 
 /* Might be faster (and bigger) if the dev/ino were stored in numeric order;) */
@@ -173,6 +172,9 @@ static void putOctal(char *cp, int len, off_t value)
 
 static void chksum_and_xwrite(int fd, struct TarHeader* hp)
 {
+	/* POSIX says that checksum is done on unsigned bytes
+	 * (Sun and HP-UX gets it wrong... more details in
+	 * GNU tar source) */
 	const unsigned char *cp;
 	int chksum, size;
 
@@ -273,8 +275,8 @@ static int writeTarHeader(struct TarBallInfo *tbInfo,
 					tbInfo->hlInfo->name, 0);
 #endif
 	} else if (S_ISLNK(statbuf->st_mode)) {
-		char *lpath = xreadlink(fileName);
-		if (!lpath)		/* Already printed err msg inside xreadlink() */
+		char *lpath = xmalloc_readlink_or_warn(fileName);
+		if (!lpath)
 			return FALSE;
 		header.typeflag = SYMTYPE;
 		strncpy(header.linkname, lpath, sizeof(header.linkname));
@@ -438,9 +440,8 @@ static int writeFileToTarball(const char *fileName, struct stat *statbuf,
 	/* Is this a regular file? */
 	if (tbInfo->hlInfo == NULL && S_ISREG(statbuf->st_mode)) {
 		/* open the file we want to archive, and make sure all is well */
-		inputFileFd = open(fileName, O_RDONLY);
+		inputFileFd = open_or_warn(fileName, O_RDONLY);
 		if (inputFileFd < 0) {
-			bb_perror_msg("%s: cannot open", fileName);
 			return FALSE;
 		}
 	}
@@ -453,7 +454,7 @@ static int writeFileToTarball(const char *fileName, struct stat *statbuf,
 	/* If it was a regular file, write out the body */
 	if (inputFileFd >= 0) {
 		size_t readSize;
-		/* Wwrite the file to the archive. */
+		/* Write the file to the archive. */
 		/* We record size into header first, */
 		/* and then write out file. If file shrinks in between, */
 		/* tar will be corrupted. So we don't allow for that. */
@@ -504,10 +505,10 @@ static int writeTarFile(const int tar_fd, const int verboseFlag,
 		int gzipDataPipe[2] = { -1, -1 };
 		int gzipStatusPipe[2] = { -1, -1 };
 		volatile int vfork_exec_errno = 0;
-		char *zip_exec = (gzip == 1) ? "gzip" : "bzip2";
+		const char *zip_exec = (gzip == 1) ? "gzip" : "bzip2";
 
-		if (pipe(gzipDataPipe) < 0 || pipe(gzipStatusPipe) < 0)
-			bb_perror_msg_and_die("pipe");
+		xpipe(gzipDataPipe);
+		xpipe(gzipStatusPipe);
 
 		signal(SIGPIPE, SIG_IGN); /* we only want EPIPE on errors */
 
@@ -529,7 +530,7 @@ static int writeTarFile(const int tar_fd, const int verboseFlag,
 			close(gzipStatusPipe[0]);
 			fcntl(gzipStatusPipe[1], F_SETFD, FD_CLOEXEC);	/* close on exec shows success */
 
-			execlp(zip_exec, zip_exec, "-f", NULL);
+			BB_EXECLP(zip_exec, zip_exec, "-f", NULL);
 			vfork_exec_errno = errno;
 
 			close(gzipStatusPipe[1]);
@@ -560,8 +561,9 @@ static int writeTarFile(const int tar_fd, const int verboseFlag,
 
 	/* Read the directory/files and iterate over them one at a time */
 	while (include) {
-		if (!recursive_action(include->data, TRUE, dereferenceFlag,
-				FALSE, writeFileToTarball, writeFileToTarball, &tbInfo, 0))
+		if (!recursive_action(include->data, ACTION_RECURSE |
+				(dereferenceFlag ? ACTION_FOLLOWLINKS : 0),
+				writeFileToTarball, writeFileToTarball, &tbInfo, 0))
 		{
 			errorFlag = TRUE;
 		}
@@ -674,7 +676,7 @@ static void handle_SIGCHLD(int status)
 		/* child exited with 0 */
 		return;
 	/* Cannot happen?
-	if(!WIFSIGNALED(status) && !WIFEXITED(status)) return; */
+	if (!WIFSIGNALED(status) && !WIFEXITED(status)) return; */
 	child_error = 1;
 }
 #endif
@@ -752,6 +754,7 @@ static const struct option tar_long_options[] = {
 };
 #endif
 
+int tar_main(int argc, char **argv);
 int tar_main(int argc, char **argv)
 {
 	char (*get_header_ptr)(archive_handle_t *) = get_header_tar;
@@ -860,10 +863,9 @@ int tar_main(int argc, char **argv)
 		char *cp = last_char_is(argv[optind], '/');
 		if (cp > argv[optind])
 			*cp = '\0';
-		llist_add_to(&tar_handle->accept, argv[optind]);
+		llist_add_to_end(&tar_handle->accept, argv[optind]);
 		optind++;
 	}
-	tar_handle->accept = rev_llist(tar_handle->accept);
 
 	if (tar_handle->accept || tar_handle->reject)
 		tar_handle->filter = filter_accept_reject_list;
