@@ -12,6 +12,7 @@
  *     modified by Herbert Xu to be used as built-in in ash.
  *     modified by Erik Andersen <andersen@codepoet.org> to be used
  *     in busybox.
+ *     modified by Bernhard Fischer to be useable (i.e. a bit less bloaty).
  *
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  *
@@ -19,13 +20,12 @@
  *     "This program is in the Public Domain."
  */
 
-#include "busybox.h"
-#include <unistd.h>
-#include <ctype.h>
-#include <errno.h>
-#include <string.h>
+#include "libbb.h"
 #include <setjmp.h>
 
+/* This is a NOEXEC applet. Be very careful! */
+
+
 /* test(1) accepts the following grammar:
 	oexpr	::= aexpr | aexpr "-o" oexpr ;
 	aexpr	::= nexpr | nexpr "-a" aexpr ;
@@ -85,7 +85,12 @@ enum token {
 	RPAREN,
 	OPERAND
 };
-
+#define is_int_op(a) (((unsigned char)((a) - INTEQ)) <= 5)
+#define is_str_op(a) (((unsigned char)((a) - STREZ)) <= 5)
+#define is_file_op(a) (((unsigned char)((a) - FILNT)) <= 2)
+#define is_file_access(a) (((unsigned char)((a) - FILRD)) <= 2)
+#define is_file_type(a) (((unsigned char)((a) - FILREG)) <= 5)
+#define is_file_bit(a) (((unsigned char)((a) - FILSUID)) <= 2)
 enum token_types {
 	UNOP,
 	BINOP,
@@ -95,64 +100,67 @@ enum token_types {
 };
 
 static const struct t_op {
-	const char *op_text;
-	short op_num, op_type;
+	char op_text[4];
+	unsigned char op_num, op_type;
 } ops[] = {
-	{
-	"-r", FILRD, UNOP}, {
-	"-w", FILWR, UNOP}, {
-	"-x", FILEX, UNOP}, {
-	"-e", FILEXIST, UNOP}, {
-	"-f", FILREG, UNOP}, {
-	"-d", FILDIR, UNOP}, {
-	"-c", FILCDEV, UNOP}, {
-	"-b", FILBDEV, UNOP}, {
-	"-p", FILFIFO, UNOP}, {
-	"-u", FILSUID, UNOP}, {
-	"-g", FILSGID, UNOP}, {
-	"-k", FILSTCK, UNOP}, {
-	"-s", FILGZ, UNOP}, {
-	"-t", FILTT, UNOP}, {
-	"-z", STREZ, UNOP}, {
-	"-n", STRNZ, UNOP}, {
-	"-h", FILSYM, UNOP},    /* for backwards compat */
-	{
-	"-O", FILUID, UNOP}, {
-	"-G", FILGID, UNOP}, {
-	"-L", FILSYM, UNOP}, {
-	"-S", FILSOCK, UNOP}, {
-	"=", STREQ, BINOP}, {
-	"==", STREQ, BINOP}, {
-	"!=", STRNE, BINOP}, {
-	"<", STRLT, BINOP}, {
-	">", STRGT, BINOP}, {
-	"-eq", INTEQ, BINOP}, {
-	"-ne", INTNE, BINOP}, {
-	"-ge", INTGE, BINOP}, {
-	"-gt", INTGT, BINOP}, {
-	"-le", INTLE, BINOP}, {
-	"-lt", INTLT, BINOP}, {
-	"-nt", FILNT, BINOP}, {
-	"-ot", FILOT, BINOP}, {
-	"-ef", FILEQ, BINOP}, {
-	"!", UNOT, BUNOP}, {
-	"-a", BAND, BBINOP}, {
-	"-o", BOR, BBINOP}, {
-	"(", LPAREN, PAREN}, {
-	")", RPAREN, PAREN}, {
-	0, 0, 0}
+	{ "-r", FILRD   , UNOP   },
+	{ "-w", FILWR   , UNOP   },
+	{ "-x", FILEX   , UNOP   },
+	{ "-e", FILEXIST, UNOP   },
+	{ "-f", FILREG  , UNOP   },
+	{ "-d", FILDIR  , UNOP   },
+	{ "-c", FILCDEV , UNOP   },
+	{ "-b", FILBDEV , UNOP   },
+	{ "-p", FILFIFO , UNOP   },
+	{ "-u", FILSUID , UNOP   },
+	{ "-g", FILSGID , UNOP   },
+	{ "-k", FILSTCK , UNOP   },
+	{ "-s", FILGZ   , UNOP   },
+	{ "-t", FILTT   , UNOP   },
+	{ "-z", STREZ   , UNOP   },
+	{ "-n", STRNZ   , UNOP   },
+	{ "-h", FILSYM  , UNOP   },    /* for backwards compat */
+
+	{ "-O" , FILUID , UNOP   },
+	{ "-G" , FILGID , UNOP   },
+	{ "-L" , FILSYM , UNOP   },
+	{ "-S" , FILSOCK, UNOP   },
+	{ "="  , STREQ  , BINOP  },
+	{ "==" , STREQ  , BINOP  },
+	{ "!=" , STRNE  , BINOP  },
+	{ "<"  , STRLT  , BINOP  },
+	{ ">"  , STRGT  , BINOP  },
+	{ "-eq", INTEQ  , BINOP  },
+	{ "-ne", INTNE  , BINOP  },
+	{ "-ge", INTGE  , BINOP  },
+	{ "-gt", INTGT  , BINOP  },
+	{ "-le", INTLE  , BINOP  },
+	{ "-lt", INTLT  , BINOP  },
+	{ "-nt", FILNT  , BINOP  },
+	{ "-ot", FILOT  , BINOP  },
+	{ "-ef", FILEQ  , BINOP  },
+	{ "!"  , UNOT   , BUNOP  },
+	{ "-a" , BAND   , BBINOP },
+	{ "-o" , BOR    , BBINOP },
+	{ "("  , LPAREN , PAREN  },
+	{ ")"  , RPAREN , PAREN  },
 };
 
-#ifdef CONFIG_FEATURE_TEST_64
+enum { NUM_OPS = sizeof(ops) / sizeof(ops[0]) };
+
+#if ENABLE_FEATURE_TEST_64
 typedef int64_t arith_t;
 #else
 typedef int arith_t;
 #endif
 
+/* Cannot eliminate these static data (do the G trick)
+ * because of bb_test usage from other applets */
 static char **t_wp;
 static struct t_op const *t_wp_op;
 static gid_t *group_array;
 static int ngroups;
+static jmp_buf leaving;
 
 static enum token t_lex(char *s);
 static arith_t oexpr(enum token n);
@@ -162,15 +170,15 @@ static int binop(void);
 static arith_t primary(enum token n);
 static int filstat(char *nm, enum token mode);
 static arith_t getn(const char *s);
+/* UNUSED
 static int newerf(const char *f1, const char *f2);
 static int olderf(const char *f1, const char *f2);
 static int equalf(const char *f1, const char *f2);
+*/
 static int test_eaccess(char *path, int mode);
 static int is_a_group_member(gid_t gid);
 static void initialize_group_array(void);
 
-static jmp_buf leaving;
-
 int bb_test(int argc, char **argv)
 {
 	int res;
@@ -203,37 +211,25 @@ int bb_test(int argc, char **argv)
 	 * isn't likely in the case of a shell.  paranoia
 	 * prevails...
 	 */
-	 ngroups = 0;
+	ngroups = 0;
 
 	/* Implement special cases from POSIX.2, section 4.62.4 */
-	switch (argc) {
-	case 1:
+	if (argc == 1)
 		return 1;
-	case 2:
+	if (argc == 2)
 		return *argv[1] == '\0';
-	case 3:
-		if (argv[1][0] == '!' && argv[1][1] == '\0') {
+//assert(argc);
+	if (LONE_CHAR(argv[1], '!')) {
+		bool _off;
+		if (argc == 3)
 			return *argv[2] != '\0';
+		_off = argc - 4;
+		t_lex(argv[2 + _off]);
+		if (t_wp_op && t_wp_op->op_type == BINOP) {
+			t_wp = &argv[1 + _off];
+			return binop() == 0;
 		}
-		break;
-	case 4:
-		if (argv[1][0] != '!' || argv[1][1] != '\0') {
-			if (t_lex(argv[2]), t_wp_op && t_wp_op->op_type == BINOP) {
-				t_wp = &argv[1];
-				return binop() == 0;
-			}
-		}
-		break;
-	case 5:
-		if (argv[1][0] == '!' && argv[1][1] == '\0') {
-			if (t_lex(argv[3]), t_wp_op && t_wp_op->op_type == BINOP) {
-				t_wp = &argv[2];
-				return binop() != 0;
-			}
-		}
-		break;
 	}
-
 	t_wp = &argv[1];
 	res = !oexpr(t_lex(*t_wp));
 
@@ -244,12 +240,13 @@ int bb_test(int argc, char **argv)
 	return res;
 }
 
+static void syntax(const char *op, const char *msg) ATTRIBUTE_NORETURN;
 static void syntax(const char *op, const char *msg)
 {
 	if (op && *op) {
 		bb_error_msg("%s: %s", op, msg);
 	} else {
-		bb_error_msg("%s", msg);
+		bb_error_msg("%s: %s"+4, msg);
 	}
 	longjmp(leaving, 2);
 }
@@ -301,73 +298,87 @@ static arith_t primary(enum token n)
 		/* unary expression */
 		if (*++t_wp == NULL)
 			syntax(t_wp_op->op_text, "argument expected");
-		switch (n) {
-		case STREZ:
-			return strlen(*t_wp) == 0;
-		case STRNZ:
-			return strlen(*t_wp) != 0;
-		case FILTT:
+		if (n == STREZ)
+			return t_wp[0][0] == '\0';
+		if (n == STRNZ)
+			return t_wp[0][0] != '\0';
+		if (n == FILTT)
 			return isatty(getn(*t_wp));
-		default:
-			return filstat(*t_wp, n);
-		}
+		return filstat(*t_wp, n);
 	}
 
-	if (t_lex(t_wp[1]), t_wp_op && t_wp_op->op_type == BINOP) {
+	t_lex(t_wp[1]);
+	if (t_wp_op && t_wp_op->op_type == BINOP) {
 		return binop();
 	}
 
-	return strlen(*t_wp) > 0;
+	return t_wp[0][0] != '\0';
 }
 
 static int binop(void)
 {
 	const char *opnd1, *opnd2;
 	struct t_op const *op;
+	arith_t val1, val2;
 
 	opnd1 = *t_wp;
 	(void) t_lex(*++t_wp);
 	op = t_wp_op;
 
-	if ((opnd2 = *++t_wp) == (char *) 0)
+	opnd2 = *++t_wp;
+	if (opnd2 == NULL)
 		syntax(op->op_text, "argument expected");
 
-	switch (op->op_num) {
-	case STREQ:
-		return strcmp(opnd1, opnd2) == 0;
-	case STRNE:
-		return strcmp(opnd1, opnd2) != 0;
-	case STRLT:
-		return strcmp(opnd1, opnd2) < 0;
-	case STRGT:
-		return strcmp(opnd1, opnd2) > 0;
-	case INTEQ:
-		return getn(opnd1) == getn(opnd2);
-	case INTNE:
-		return getn(opnd1) != getn(opnd2);
-	case INTGE:
-		return getn(opnd1) >= getn(opnd2);
-	case INTGT:
-		return getn(opnd1) > getn(opnd2);
-	case INTLE:
-		return getn(opnd1) <= getn(opnd2);
-	case INTLT:
-		return getn(opnd1) < getn(opnd2);
-	case FILNT:
-		return newerf(opnd1, opnd2);
-	case FILOT:
-		return olderf(opnd1, opnd2);
-	case FILEQ:
-		return equalf(opnd1, opnd2);
+	if (is_int_op(op->op_num)) {
+		val1 = getn(opnd1);
+		val2 = getn(opnd2);
+		if (op->op_num == INTEQ)
+			return val1 == val2;
+		if (op->op_num == INTNE)
+			return val1 != val2;
+		if (op->op_num == INTGE)
+			return val1 >= val2;
+		if (op->op_num == INTGT)
+			return val1 >  val2;
+		if (op->op_num == INTLE)
+			return val1 <= val2;
+		if (op->op_num == INTLT)
+			return val1 <  val2;
 	}
-	/* NOTREACHED */
-	return 1;
+	if (is_str_op(op->op_num)) {
+		val1 = strcmp(opnd1, opnd2);
+		if (op->op_num == STREQ)
+			return val1 == 0;
+		if (op->op_num == STRNE)
+			return val1 != 0;
+		if (op->op_num == STRLT)
+			return val1 < 0;
+		if (op->op_num == STRGT)
+			return val1 > 0;
+	}
+	/* We are sure that these three are by now the only binops we didn't check
+	 * yet, so we do not check if the class is correct:
+	 */
+/*	if (is_file_op(op->op_num)) */
+	{
+		struct stat b1, b2;
+
+		if (stat(opnd1, &b1) || stat(opnd2, &b2))
+			return 0; /* false, since at least one stat failed */
+		if (op->op_num == FILNT)
+			return b1.st_mtime > b2.st_mtime;
+		if (op->op_num == FILOT)
+			return b1.st_mtime < b2.st_mtime;
+		if (op->op_num == FILEQ)
+			return b1.st_dev == b2.st_dev && b1.st_ino == b2.st_ino;
+	}
+	return 1; /* NOTREACHED */
 }
 
 static int filstat(char *nm, enum token mode)
 {
 	struct stat s;
-	unsigned int i;
+	int i;
 
 	if (mode == FILSYM) {
 #ifdef S_IFLNK
@@ -381,99 +392,95 @@ static int filstat(char *nm, enum token mode)
 
 	if (stat(nm, &s) != 0)
 		return 0;
-
-	switch (mode) {
-	case FILRD:
-		return test_eaccess(nm, R_OK) == 0;
-	case FILWR:
-		return test_eaccess(nm, W_OK) == 0;
-	case FILEX:
-		return test_eaccess(nm, X_OK) == 0;
-	case FILEXIST:
-		return 1;
-	case FILREG:
-		i = S_IFREG;
-		goto filetype;
-	case FILDIR:
-		i = S_IFDIR;
-		goto filetype;
-	case FILCDEV:
-		i = S_IFCHR;
-		goto filetype;
-	case FILBDEV:
-		i = S_IFBLK;
-		goto filetype;
-	case FILFIFO:
-#ifdef S_IFIFO
-		i = S_IFIFO;
-		goto filetype;
-#else
-		return 0;
-#endif
-	case FILSOCK:
-#ifdef S_IFSOCK
-		i = S_IFSOCK;
-		goto filetype;
-#else
-		return 0;
-#endif
-	case FILSUID:
-		i = S_ISUID;
-		goto filebit;
-	case FILSGID:
-		i = S_ISGID;
-		goto filebit;
-	case FILSTCK:
-		i = S_ISVTX;
-		goto filebit;
-	case FILGZ:
-		return s.st_size > 0L;
-	case FILUID:
-		return s.st_uid == geteuid();
-	case FILGID:
-		return s.st_gid == getegid();
-	default:
+	if (mode == FILEXIST)
 		return 1;
+	if (is_file_access(mode)) {
+		if (mode == FILRD)
+			i = R_OK;
+		if (mode == FILWR)
+			i = W_OK;
+		if (mode == FILEX)
+			i = X_OK;
+		return test_eaccess(nm, i) == 0;
 	}
-
-  filetype:
-	return ((s.st_mode & S_IFMT) == i);
-
-  filebit:
-	return ((s.st_mode & i) != 0);
+	if (is_file_type(mode)) {
+		if (mode == FILREG)
+			i = S_IFREG;
+		if (mode == FILDIR)
+			i = S_IFDIR;
+		if (mode == FILCDEV)
+			i = S_IFCHR;
+		if (mode == FILBDEV)
+			i = S_IFBLK;
+		if (mode == FILFIFO) {
+#ifdef S_IFIFO
+			i = S_IFIFO;
+#else
+			return 0;
+#endif
+		}
+		if (mode == FILSOCK) {
+#ifdef S_IFSOCK
+			i = S_IFSOCK;
+#else
+			return 0;
+#endif
+		}
+ filetype:
+		return ((s.st_mode & S_IFMT) == i);
+	}
+	if (is_file_bit(mode)) {
+		if (mode == FILSUID)
+			i = S_ISUID;
+		if (mode == FILSGID)
+			i = S_ISGID;
+		if (mode == FILSTCK)
+			i = S_ISVTX;
+		return ((s.st_mode & i) != 0);
+	}
+	if (mode == FILGZ)
+		return s.st_size > 0L;
+	if (mode == FILUID)
+		return s.st_uid == geteuid();
+	if (mode == FILGID)
+		return s.st_gid == getegid();
+	return 1; /* NOTREACHED */
 }
 
 static enum token t_lex(char *s)
 {
-	struct t_op const *op = ops;
+	const struct t_op *op;
 
-	if (s == 0) {
-		t_wp_op = (struct t_op *) 0;
+	t_wp_op = NULL;
+	if (s == NULL) {
 		return EOI;
 	}
-	while (op->op_text) {
+
+	op = ops;
+	do {
 		if (strcmp(s, op->op_text) == 0) {
 			t_wp_op = op;
 			return op->op_num;
 		}
 		op++;
-	}
-	t_wp_op = (struct t_op *) 0;
+	} while (op < ops + NUM_OPS);
+
 	return OPERAND;
 }
 
 /* atoi with error detection */
+//XXX: FIXME: duplicate of existing libbb function?
 static arith_t getn(const char *s)
 {
 	char *p;
-#ifdef CONFIG_FEATURE_TEST_64
+#if ENABLE_FEATURE_TEST_64
 	long long r;
 #else
 	long r;
 #endif
 
 	errno = 0;
-#ifdef CONFIG_FEATURE_TEST_64
+#if ENABLE_FEATURE_TEST_64
 	r = strtoll(s, &p, 10);
 #else
 	r = strtol(s, &p, 10);
@@ -488,6 +495,7 @@ static arith_t getn(const char *s)
 	return r;
 }
 
+/* UNUSED
 static int newerf(const char *f1, const char *f2)
 {
 	struct stat b1, b2;
@@ -512,6 +520,7 @@ static int equalf(const char *f1, const char *f2)
 			stat(f2, &b2) == 0 &&
 			b1.st_dev == b2.st_dev && b1.st_ino == b2.st_ino);
 }
+*/
 
 /* Do the same thing access(2) does, but use the effective uid and gid,
    and don't make the mistake of telling root that any file is
@@ -550,12 +559,20 @@ static void initialize_group_array(void)
 {
 	ngroups = getgroups(0, NULL);
 	if (ngroups > 0) {
-		group_array = xmalloc(ngroups * sizeof(gid_t));
+		/* FIXME: ash tries so hard to not die on OOM,
+		 * and we spoil it with just one xrealloc here */
+		/* We realloc, because bb_test can be entered repeatedly by shell.
+		 * Testcase (ash): 'while true; do test -x some_file; done'
+		 * and watch top. (some_file must have owner != you) */
+		group_array = xrealloc(group_array, ngroups * sizeof(gid_t));
 		getgroups(ngroups, group_array);
 	}
 }
 
 /* Return non-zero if GID is one that we have in our groups list. */
+//XXX: FIXME: duplicate of existing libbb function?
+// see toplevel TODO file:
+// possible code duplication ingroup() and is_a_group_member()
 static int is_a_group_member(gid_t gid)
 {
 	int i;
@@ -578,8 +595,8 @@ static int is_a_group_member(gid_t gid)
 
 /* applet entry point */
 
+int test_main(int argc, char **argv);
 int test_main(int argc, char **argv)
 {
 	return bb_test(argc, argv);
 }
-
