@@ -2,16 +2,14 @@
 /*
  * Utility routines.
  *
- * create raw socket for icmp protocol test permision
- * and drop root privilegies if running setuid
+ * create raw socket for icmp protocol test permission
+ * and drop root privileges if running setuid
  *
  */
 
 #include <sys/types.h>
 #include <netdb.h>
 #include <sys/socket.h>
-#include <errno.h>
-#include <unistd.h>
 #include "libbb.h"
 
 int create_icmp_socket(void)
@@ -22,16 +20,16 @@ int create_icmp_socket(void)
 	proto = getprotobyname("icmp");
 	/* if getprotobyname failed, just silently force
 	 * proto->p_proto to have the correct value for "icmp" */
-	if ((sock = socket(AF_INET, SOCK_RAW,
-			(proto ? proto->p_proto : 1))) < 0) {        /* 1 == ICMP */
+	sock = socket(AF_INET, SOCK_RAW,
+			(proto ? proto->p_proto : 1)); /* 1 == ICMP */
+	if (sock < 0) {
 		if (errno == EPERM)
-			error_msg_and_die("permission denied. (are you root?)");
-		else
-			perror_msg_and_die(can_not_create_raw_socket);
+			bb_error_msg_and_die(bb_msg_perm_denied_are_you_root);
+		bb_perror_msg_and_die(bb_msg_can_not_create_raw_socket);
 	}
 
 	/* drop root privs if running setuid */
-	setuid(getuid());
+	xsetuid(getuid());
 
 	return sock;
 }
