@@ -7,17 +7,16 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include "busybox.h"
-#include <stdio.h>
-#include <stdlib.h>
 #include <getopt.h>
-#include <ctype.h>
+
+#include "libbb.h"
 
 #define WHOLE_FILE		1
 #define PRINT_NAME		2
 #define PRINT_OFFSET	4
 #define SIZE			8
 
+int strings_main(int argc, char **argv);
 int strings_main(int argc, char **argv)
 {
 	int n, c, i = 0, status = EXIT_SUCCESS;
@@ -26,7 +25,7 @@ int strings_main(int argc, char **argv)
 	FILE *file = stdin;
 	char *string;
 	const char *fmt = "%s: ";
-	char *n_arg = "4";
+	const char *n_arg = "4";
 
 	opt = getopt32(argc, argv, "afon:", &n_arg);
 	/* -a is our default behaviour */
