@@ -2,19 +2,17 @@
 #ifndef __UTILS_H__
 #define __UTILS_H__ 1
 
-#include "libbb.h"
-
 #include "libnetlink.h"
 #include "ll_map.h"
 #include "rtm_map.h"
 
 extern int preferred_family;
-extern int show_stats;
-extern int show_details;
-extern int show_raw;
-extern int resolve_hosts;
-extern int oneline;
-extern char * _SL_;
+extern smallint show_stats;    /* UNUSED */
+extern smallint show_details;  /* UNUSED */
+extern smallint show_raw;      /* UNUSED */
+extern smallint resolve_hosts; /* UNUSED */
+extern smallint oneline;
+extern char _SL_;
 
 #ifndef IPPROTO_ESP
 #define IPPROTO_ESP	50
@@ -43,8 +41,7 @@ typedef struct
 #define AF_DECnet 12
 #endif
 
-struct dn_naddr
-{
+struct dn_naddr {
 	unsigned short          a_len;
 	unsigned char a_addr[DN_MAXADDL];
 };
@@ -76,11 +73,11 @@ extern int get_s8(int8_t *val, char *arg, int base);
 extern const char *format_host(int af, int len, void *addr, char *buf, int buflen);
 extern const char *rt_addr_n2a(int af, int len, void *addr, char *buf, int buflen);
 
-void invarg(const char * const, const char * const) ATTRIBUTE_NORETURN;
-void duparg(char *, char *) ATTRIBUTE_NORETURN;
-void duparg2(char *, char *) ATTRIBUTE_NORETURN;
-int matches(char *arg, char *pattern);
-extern int inet_addr_match(inet_prefix *a, inet_prefix *b, int bits);
+void invarg(const char *, const char *) ATTRIBUTE_NORETURN;
+void duparg(const char *, const char *) ATTRIBUTE_NORETURN;
+void duparg2(const char *, const char *) ATTRIBUTE_NORETURN;
+int ATTRIBUTE_DEPRECATED matches(const char *arg, const char *pattern);
+int inet_addr_match(inet_prefix *a, inet_prefix *b, int bits);
 
 const char *dnet_ntop(int af, const void *addr, char *str, size_t len);
 int dnet_pton(int af, const char *src, void *addr);
