@@ -10,8 +10,8 @@
 /* http://www.unix.org/version2/whatsnew/lfs20mar.html */
 #define _LARGEFILE64_SOURCE 1
 
-#include "busybox.h"
 #include <getopt.h>	/* for struct option */
+#include "libbb.h"
 
 struct host_info {
 	// May be used if we ever will want to free() all xstrdup()s...
@@ -26,7 +26,7 @@ struct host_info {
 static void parse_url(char *url, struct host_info *h);
 static FILE *open_socket(len_and_sockaddr *lsa);
 static char *gethdr(char *buf, size_t bufsiz, FILE *fp, int *istrunc);
-static int ftpcmd(char *s1, char *s2, FILE *fp, char *buf);
+static int ftpcmd(const char *s1, const char *s2, FILE *fp, char *buf);
 
 /* Globals (can be accessed from signal handlers */
 static off_t content_len;        /* Content-length of the file */
@@ -34,10 +34,10 @@ static off_t beg_range;          /* Range at which continue begins */
 #if ENABLE_FEATURE_WGET_STATUSBAR
 static off_t transferred;        /* Number of bytes transferred so far */
 #endif
-static int chunked;                     /* chunked transfer encoding */
+static bool chunked;                     /* chunked transfer encoding */
 #if ENABLE_FEATURE_WGET_STATUSBAR
 static void progressmeter(int flag);
-static char *curfile;                   /* Name of current file being transferred */
+static const char *curfile;             /* Name of current file being transferred */
 static struct timeval start;            /* Time a transfer started */
 enum {
 	STALLTIME = 5                   /* Seconds when xfer considered "stalled" */
@@ -76,9 +76,7 @@ static char *safe_fgets(char *s, int size, FILE *stream)
 }
 
 #if ENABLE_FEATURE_WGET_AUTHENTICATION
-/*
- *  Base64-encode character string and return the string.
- */
+/* Base64-encode character string and return the string.  */
 static char *base64enc(unsigned char *p, char *buf, int len)
 {
 	bb_uuencode(p, buf, len, bb_uuenc_tbl_base64);
@@ -86,6 +84,7 @@ static char *base64enc(unsigned char *p, char *buf, int len)
 }
 #endif
 
+int wget_main(int argc, char **argv);
 int wget_main(int argc, char **argv)
 {
 	char buf[512];
@@ -95,7 +94,7 @@ int wget_main(int argc, char **argv)
 	int port;
 	int try = 5;
 	unsigned opt;
-	char *s;
+	char *str;
 	char *proxy = 0;
 	char *dir_prefix = NULL;
 #if ENABLE_FEATURE_WGET_LONG_OPTIONS
@@ -103,34 +102,36 @@ int wget_main(int argc, char **argv)
 	llist_t *headers_llist = NULL;
 #endif
 
-	/* server.allocated = target.allocated = NULL; */
-
 	FILE *sfp = NULL;               /* socket to web/ftp server         */
 	FILE *dfp = NULL;               /* socket to ftp server (data)      */
 	char *fname_out = NULL;         /* where to direct output (-O)      */
-	int got_clen = 0;               /* got content-length: from server  */
+	bool got_clen = 0;               /* got content-length: from server  */
 	int output_fd = -1;
-	int use_proxy = 1;              /* Use proxies if env vars are set  */
+	bool use_proxy = 1;              /* Use proxies if env vars are set  */
 	const char *proxy_flag = "on";  /* Use proxies if env vars are set  */
-	const char *user_agent = "Wget";/* Content of the "User-Agent" header field */
-
-	/*
-	 * Crack command line.
-	 */
+	const char *user_agent = "Wget";/* "User-Agent" header field        */
+	static const char * const keywords[] = {
+		"content-length", "transfer-encoding", "chunked", "location", NULL
+	};
+	enum {
+		KEY_content_length = 1, KEY_transfer_encoding, KEY_chunked, KEY_location
+	};
 	enum {
 		WGET_OPT_CONTINUE   = 0x1,
-		WGET_OPT_QUIET      = 0x2,
-		WGET_OPT_OUTNAME    = 0x4,
-		WGET_OPT_PREFIX     = 0x8,
-		WGET_OPT_PROXY      = 0x10,
-		WGET_OPT_USER_AGENT = 0x20,
-		WGET_OPT_PASSIVE    = 0x40,
-		WGET_OPT_HEADER     = 0x80,
+		WGET_OPT_SPIDER	    = 0x2,
+		WGET_OPT_QUIET      = 0x4,
+		WGET_OPT_OUTNAME    = 0x8,
+		WGET_OPT_PREFIX     = 0x10,
+		WGET_OPT_PROXY      = 0x20,
+		WGET_OPT_USER_AGENT = 0x40,
+		WGET_OPT_PASSIVE    = 0x80,
+		WGET_OPT_HEADER     = 0x100,
 	};
 #if ENABLE_FEATURE_WGET_LONG_OPTIONS
 	static const struct option wget_long_options[] = {
-		// name, has_arg, flag, val
+		/* name, has_arg, flag, val */
 		{ "continue",         no_argument, NULL, 'c' },
+		{ "spider",           no_argument, NULL, 's' },
 		{ "quiet",            no_argument, NULL, 'q' },
 		{ "output-document",  required_argument, NULL, 'O' },
 		{ "directory-prefix", required_argument, NULL, 'P' },
@@ -142,21 +143,22 @@ int wget_main(int argc, char **argv)
 	};
 	applet_long_options = wget_long_options;
 #endif
+	/* server.allocated = target.allocated = NULL; */
 	opt_complementary = "-1" USE_FEATURE_WGET_LONG_OPTIONS(":\xfe::");
-	opt = getopt32(argc, argv, "cqO:P:Y:U:",
+	opt = getopt32(argc, argv, "csqO:P:Y:U:",
 				&fname_out, &dir_prefix,
 				&proxy_flag, &user_agent
 				USE_FEATURE_WGET_LONG_OPTIONS(, &headers_llist)
 				);
 	if (strcmp(proxy_flag, "off") == 0) {
-		/* Use the proxy if necessary. */
+		/* Use the proxy if necessary */
 		use_proxy = 0;
 	}
 #if ENABLE_FEATURE_WGET_LONG_OPTIONS
 	if (headers_llist) {
 		int size = 1;
 		char *cp;
-		llist_t *ll = headers_llist = rev_llist(headers_llist);
+		llist_t *ll = headers_llist;
 		while (ll) {
 			size += strlen(ll->data) + 2;
 			ll = ll->link;
@@ -173,9 +175,7 @@ int wget_main(int argc, char **argv)
 	server.host = target.host;
 	server.port = target.port;
 
-	/*
-	 * Use the proxy if necessary.
-	 */
+	/* Use the proxy if necessary */
 	if (use_proxy) {
 		proxy = getenv(target.is_ftp ? "ftp_proxy" : "http_proxy");
 		if (proxy && *proxy) {
@@ -190,18 +190,18 @@ int wget_main(int argc, char **argv)
 		// Dirty hack. Needed because bb_get_last_path_component
 		// will destroy trailing / by storing '\0' in last byte!
 		if (!last_char_is(target.path, '/')) {
-			fname_out =
+			fname_out = bb_get_last_path_component(target.path);
 #if ENABLE_FEATURE_WGET_STATUSBAR
-				curfile =
+			curfile = fname_out;
 #endif
-				bb_get_last_path_component(target.path);
 		}
 		if (!fname_out || !fname_out[0]) {
-			fname_out =
+			/* bb_get_last_path_component writes
+			 * to last '/' only. We don't have one here... */
+			fname_out = (char*)"index.html";
 #if ENABLE_FEATURE_WGET_STATUSBAR
-				curfile =
+			curfile = fname_out;
 #endif
-				"index.html";
 		}
 		if (dir_prefix != NULL)
 			fname_out = concat_path_file(dir_prefix, fname_out);
@@ -214,9 +214,7 @@ int wget_main(int argc, char **argv)
 	if ((opt & WGET_OPT_CONTINUE) && !fname_out)
 		bb_error_msg_and_die("cannot specify continue (-c) without a filename (-O)"); */
 
-	/*
-	 * Determine where to start transfer.
-	 */
+	/* Determine where to start transfer */
 	if (LONE_DASH(fname_out)) {
 		output_fd = 1;
 		opt &= ~WGET_OPT_CONTINUE;
@@ -233,11 +231,11 @@ int wget_main(int argc, char **argv)
 	/* We want to do exactly _one_ DNS lookup, since some
 	 * sites (i.e. ftp.us.debian.org) use round-robin DNS
 	 * and we want to connect to only one IP... */
-	lsa = host2sockaddr(server.host, server.port);
+	lsa = xhost2sockaddr(server.host, server.port);
 	if (!(opt & WGET_OPT_QUIET)) {
-		fprintf(stderr, "Connecting to %s [%s]\n", server.host,
+		fprintf(stderr, "Connecting to %s (%s)\n", server.host,
 				xmalloc_sockaddr2dotted(&lsa->sa, lsa->len));
-		/* We leak xmalloc_sockaddr2dotted result */
+		/* We leak result of xmalloc_sockaddr2dotted */
 	}
 
 	if (use_proxy || !target.is_ftp) {
@@ -250,24 +248,12 @@ int wget_main(int argc, char **argv)
 			if (!--try)
 				bb_error_msg_and_die("too many redirections");
 
-			/*
-			 * Open socket to http server
-			 */
+			/* Open socket to http server */
 			if (sfp) fclose(sfp);
 			sfp = open_socket(lsa);
 
-			/*
-			 * Send HTTP request.
-			 */
+			/* Send HTTP request.  */
 			if (use_proxy) {
-//				const char *format = "GET %stp://%s:%d/%s HTTP/1.1\r\n";
-//#if ENABLE_FEATURE_WGET_IP6_LITERAL
-//				if (strchr(target.host, ':'))
-//					format = "GET %stp://[%s]:%d/%s HTTP/1.1\r\n";
-//#endif
-//				fprintf(sfp, format,
-//					target.is_ftp ? "f" : "ht", target.host,
-//					ntohs(target.port), target.path);
 				fprintf(sfp, "GET %stp://%s/%s HTTP/1.1\r\n",
 					target.is_ftp ? "f" : "ht", target.host,
 					target.path);
@@ -304,12 +290,12 @@ int wget_main(int argc, char **argv)
 			if (fgets(buf, sizeof(buf), sfp) == NULL)
 				bb_error_msg_and_die("no response from server");
 
-			s = buf;
-			while (*s != '\0' && !isspace(*s)) ++s;
-			s = skip_whitespace(s);
+			str = buf;
+			str = skip_non_whitespace(str);
+			str = skip_whitespace(str);
 			// FIXME: no error check
 			// xatou wouldn't work: "200 OK"
-			status = atoi(s);
+			status = atoi(str);
 			switch (status) {
 			case 0:
 			case 100:
@@ -336,32 +322,34 @@ int wget_main(int argc, char **argv)
 			/*
 			 * Retrieve HTTP headers.
 			 */
-			while ((s = gethdr(buf, sizeof(buf), sfp, &n)) != NULL) {
-				if (strcasecmp(buf, "content-length") == 0) {
-					content_len = BB_STRTOOFF(s, NULL, 10);
+			while ((str = gethdr(buf, sizeof(buf), sfp, &n)) != NULL) {
+				/* gethdr did already convert the "FOO:" string to lowercase */
+				smalluint key = index_in_str_array(keywords, *&buf) + 1;
+				if (key == KEY_content_length) {
+					content_len = BB_STRTOOFF(str, NULL, 10);
 					if (errno || content_len < 0) {
-						bb_error_msg_and_die("content-length %s is garbage", s);
+						bb_error_msg_and_die("content-length %s is garbage", str);
 					}
 					got_clen = 1;
 					continue;
 				}
-				if (strcasecmp(buf, "transfer-encoding") == 0) {
-					if (strcasecmp(s, "chunked") != 0)
-						bb_error_msg_and_die("server wants to do %s transfer encoding", s);
+				if (key == KEY_transfer_encoding) {
+					if (index_in_str_array(keywords, str_tolower(str)) + 1 != KEY_chunked)
+						bb_error_msg_and_die("server wants to do %s transfer encoding", str);
 					chunked = got_clen = 1;
 				}
-				if (strcasecmp(buf, "location") == 0) {
-					if (s[0] == '/')
+				if (key == KEY_location) {
+					if (str[0] == '/')
 						/* free(target.allocated); */
-						target.path = /* target.allocated = */ xstrdup(s+1);
+						target.path = /* target.allocated = */ xstrdup(str+1);
 					else {
-						parse_url(s, &target);
+						parse_url(str, &target);
 						if (use_proxy == 0) {
 							server.host = target.host;
 							server.port = target.port;
 						}
 						free(lsa);
-						lsa = host2sockaddr(server.host, server.port);
+						lsa = xhost2sockaddr(server.host, server.port);
 						break;
 					}
 				}
@@ -386,14 +374,14 @@ int wget_main(int argc, char **argv)
 		 * Splitting username:password pair,
 		 * trying to log in
 		 */
-		s = strchr(target.user, ':');
-		if (s)
-			*(s++) = '\0';
+		str = strchr(target.user, ':');
+		if (str)
+			*(str++) = '\0';
 		switch (ftpcmd("USER ", target.user, sfp, buf)) {
 		case 230:
 			break;
 		case 331:
-			if (ftpcmd("PASS ", s, sfp, buf) == 230)
+			if (ftpcmd("PASS ", str, sfp, buf) == 230)
 				break;
 			/* FALLTHRU (failed login) */
 		default:
@@ -423,15 +411,15 @@ int wget_main(int argc, char **argv)
 		// Response is "227 garbageN1,N2,N3,N4,P1,P2[)garbage]
 		// Server's IP is N1.N2.N3.N4 (we ignore it)
 		// Server's port for data connection is P1*256+P2
-		s = strrchr(buf, ')');
-		if (s) s[0] = '\0';
-		s = strrchr(buf, ',');
-		if (!s) goto pasv_error;
-		port = xatou_range(s+1, 0, 255);
-		*s = '\0';
-		s = strrchr(buf, ',');
-		if (!s) goto pasv_error;
-		port += xatou_range(s+1, 0, 255) * 256;
+		str = strrchr(buf, ')');
+		if (str) str[0] = '\0';
+		str = strrchr(buf, ',');
+		if (!str) goto pasv_error;
+		port = xatou_range(str+1, 0, 255);
+		*str = '\0';
+		str = strrchr(buf, ',');
+		if (!str) goto pasv_error;
+		port += xatou_range(str+1, 0, 255) * 256;
 		set_nport(lsa, htons(port));
 		dfp = open_socket(lsa);
 
@@ -444,7 +432,11 @@ int wget_main(int argc, char **argv)
 		if (ftpcmd("RETR ", target.path, sfp, buf) > 150)
 			bb_error_msg_and_die("bad response to RETR: %s", buf);
 	}
-
+	if (opt & WGET_OPT_SPIDER) {
+		if (ENABLE_FEATURE_CLEAN_UP)
+			fclose(sfp);
+		goto done;
+	}
 
 	/*
 	 * Retrieve file
@@ -506,6 +498,7 @@ int wget_main(int argc, char **argv)
 			bb_error_msg_and_die("ftp error: %s", buf+4);
 		ftpcmd("QUIT", NULL, sfp, buf);
 	}
+done:
 	exit(EXIT_SUCCESS);
 }
 
@@ -568,28 +561,6 @@ static void parse_url(char *src_url, struct host_info *h)
 	}
 
 	sp = h->host;
-
-//host2sockaddr does this itself
-//#if ENABLE_FEATURE_WGET_IP6_LITERAL
-//	if (sp[0] == '[') {
-//		char *ep;
-//
-//		ep = sp + 1;
-//		while (*ep == ':' || isxdigit(*ep))
-//			ep++;
-//		if (*ep == ']') {
-//			h->host++;
-//			*ep = '\0';
-//			sp = ep + 1;
-//		}
-//	}
-//#endif
-//
-//	p = strchr(sp, ':');
-//	if (p != NULL) {
-//		*p = '\0';
-//		h->port = htons(xatou16(p + 1));
-//	}
 }
 
 
@@ -654,7 +625,7 @@ static char *gethdr(char *buf, size_t bufsiz, FILE *fp, int *istrunc)
 	return hdrval;
 }
 
-static int ftpcmd(char *s1, char *s2, FILE *fp, char *buf)
+static int ftpcmd(const char *s1, const char *s2, FILE *fp, char *buf)
 {
 	int result;
 	if (s1) {
@@ -834,5 +805,4 @@ progressmeter(int flag)
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: wget.c,v 1.75 2004/10/08 08:27:40 andersen Exp $
  */
